/**
 * Ein Objekt einer konkreten Unterklasse dieser Klasse
 * reprsentiert einen Ausdruck, der basierend auf einer
 * Variablenbelegung ausgewertet werden kann.
 */
public abstract class Ausdruck {
  
  /**
   * Stellt diesen Ausdruck in Postfix-Darstellung dar.
   *
   * @return Postfix-Darstellung dieses Ausdrucks
   */
  public abstract String postfixDarstellung();
  
  /**
   * Wertet diesen Ausdruck auf der Basis der angegebenen
   * Variablenbelegung aus.
   *
   * @param belegung  Variablenbelegung, auf deren Basis
   *        dieser Ausdruck ausgewertet wird
   * @return Wert dieses Ausdrucks
   * @throws UnbekannteVariableException  wird geworfen, wenn
   *         eine Variable des Ausdrucks in der Belegung keinen
   *         zugeordneten Wert besitzt
   */
  public abstract int gibWert(Variablenbelegung belegung)
          throws UnbekannteVariableException;
  
}
