/*
 * Christoph Peters
 */

import java.text.ParseException;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;


/**
 * Testklasse basiernd auf Junit fr die Methoden der Klasse Parser.
 */
public class ParserTest extends TestCase {
       
    /** Instanzvariable fr Parser. */
    private Parser parser;
    
    /**
     * Erzeugt eine neue Instanz dieser Klasse
     * mit dem angegebenen Namen.
     *
     * @param testName Name dieses Tests.
     */
    public ParserTest(String testName) {
        super(testName);
    }
    
    /**
     * Liefert Suite der Testmethoden dieser Klasse.
     *
     * @return  Testmethoden dieser Klasse.
     */
    public static Test suite() {
        
        return new TestSuite(ParserTest.class);
    }
    
    /**
     * Testdaten aufbauen.
     */
    protected void setUp() {
        parser = new Parser();
    }
    
    /**
     * Testdaten abbauen.
     */
    protected void tearDown() {
    }

    /**
     * Diese Methode testet ob die postfixDarstellung richtig zurck gegeben
     * wird.
     *
     * @throws ParseException  Weitergegebene Exception der Methode
     *                         Parser.parse.
     */
    public void testParsePostfixDarstellung() throws ParseException {
        
        assertEquals("10", parser.parse("10").postfixDarstellung());
        assertEquals("betrag", parser.parse("betrag").postfixDarstellung());
        assertEquals("10 betrag *",
                parser.parse(" 10 * betrag ").postfixDarstellung());
        assertEquals("10 betrag * anzahl /",
                parser.parse("10*betrag/anzahl").postfixDarstellung());
        assertEquals("1 2 i 2 k * + 1 - * 2 / + j +",
                parser.parse("1 + 2 * (i + 2 * k - 1) / 2 + j")
                            .postfixDarstellung());
    }
  
    /**
     * Die Methode testParseException() beinhaltet Testflle mit 
     * aufrufen zum Testen.
     */
    public void testParseException() {
        
        tokenTesten("1a", "1a", 0);
        tokenTesten("2 - )", ")", 2);     
        tokenTesten("2 * (s - t * a_b)", "a_b", 7);
        tokenTesten("2 * (s - )", ")", 5);
        
//        /* Eigene Testflle */
//        tokenTesten("2 6", "6", 1);
//        tokenTesten("(2 6)", "6", 2);
//        tokenTesten("(2+6) &", "&", 5);
//        tokenTesten("&(2+6)", "&", 0);
//        tokenTesten("(2))", ")", 3);
//        tokenTesten("()", ")", 1);
        
        /* Testflle Blatt */
        endeTesten("2 * (s - t * 5");
        endeTesten("2 * (3 * (i + 4)");
//        /* Eigene Testflle */
//        endeTesten("(2");
//        endeTesten("(");
//        endeTesten("");
//        endeTesten("                ");
        
    }
   
  /**
   * Die Methode tokenTesten() fngt fr ungltige Flle die ParseException fr 
   * "ein ungltiges Token" ab.
   *
   * @param zeichen  Zeichen, welche geprft werden sollen.
   * @param wrongToken  Token bei dem der Fehler auftritt.
   * @param errorOffset  Stelle, an der der Fehler auftritt.
   */
  public void tokenTesten(String zeichen, String wrongToken, int errorOffset) {

    /*
     * Der Testdurchlauf muss wieder auf null gesetzt werden, da er sonst
     * falsche aufrufe erzeugen kann.
     */
      tearDown();
      /* Ein neuer Parser wird erffnet */
      setUp();
      
      try {
          parser.parse(zeichen);
          fail("Achtung: keine ParseException fr: " + wrongToken);
          
      } catch (ParseException e) {
          assertEquals("ungueltiges Token " + wrongToken, e.getMessage());
          assertEquals(errorOffset, e.getErrorOffset());
      }
  }
  
  /**
   * Die Methode endeTesten() fngt fr ungltige Flle die ParseException fr 
   * "unerwartetes Ende" ab.
   *
   * @param zeichenkette welche geparst werden soll.
   */
  public void endeTesten(String zeichenkette) {

    /*
     * Der Testdurchlauf muss wieder auf null gesetzt werden, da er sonst
     * falsche aufrufe erzeugen kann.
     */
      tearDown();
      /* Neuer Testdurchlauf aufbauen. */
      setUp();
      try {
          parser.parse(zeichenkette);
          fail("Keine ParseException geworfen fuer erwarteten Fehler "
                  + "'unerwartetes Ende'");
      } catch (ParseException e) {
          assertEquals("unerwartetes Ende", e.getMessage());
      }
  }

  /**
   * Diese Methode testet testGibWert() ob die gibWert Methode die richtigen 
   * Werte zurck gibt und auch die UnbekannteVariableException auf korrektheit.
   *
   * @throws ParseException Weitergegebene Exception der Methode
   * Parser.parse
   * @throws UnbekannteVariableException Weitergegebene Exception der
   * Methode Ausdruck.gibWert.
   */
  public void testGibWert() throws ParseException, UnbekannteVariableException {

    Variablenbelegung[] belegung = new Variablenbelegung[4];

    // Erste Belegung erstellen
    belegung[0] = new Variablenbelegung();

    // Zweite Belegung erstellen
    belegung[1] = new Variablenbelegung();
    belegung[1].belege("i", 3);

    // Dritte Belegung erstellen
    belegung[2] = new Variablenbelegung();
    belegung[2].belege("i", 3);
    belegung[2].belege("k", 4);
    belegung[2].belege("j", 9);

    // Vierte Belegung erstellen
    belegung[3] = new Variablenbelegung();
    belegung[3].belege("k", 4);
    belegung[3].belege("j", 9);

    
    // Test der UnbekannteVariableException
    unbekannteVariableTesten("i", belegung[0], "i");
    unbekannteVariableTesten("i + 2 * j", belegung[1], "j");

    // Test der gibWert Methode
    assertEquals(10, parser.parse("10").gibWert(belegung[0]));
    assertEquals(10, parser.parse("7 + i").gibWert(belegung[1]));
    assertEquals(10, parser.parse("(i + 18)/2").gibWert(belegung[1]));
    assertEquals(20,
        parser.parse("1 + 2 * (i + 2 * k - 1) / 2 + j").gibWert(belegung[2]));

    /**
     * Wenn der beschriebene Fehler auftritt und noch die korrekte
     * Fehlermeldung inbehlt, ist der Test erfolgreich.
     */
    try {
        parser.parse("2 / (j - 2 * k - 1)").gibWert(belegung[3]);
        fail("Keine ArithmeticException geworfen fuer '/ by zero'");
        
    } catch (ArithmeticException e) {
        assertEquals("/ by zero", e.getMessage());
    }
  }

  /**
   * Die Methode unbekannteVariableTesten() testet die gibWert Methode auf
   * UnbekannteVariableExceptions.
   *
   * @param parse Zeichenkette, welche geparst werden soll.
   * @param belegung Belegung fr die zu parsende Zeichenkette.
   * @param invalidVar Variable bei welchem der Fehler auftreten sollte.
   * @throws ParseException Weitergegebene Exception der Methode parse.
   * @throws UnbekannteVariableException Weitergegebene Exception der
   *                                     Methode parse.
   */
  public void unbekannteVariableTesten(String parse,
                       Variablenbelegung belegung, String invalidVar)
                          throws ParseException, UnbekannteVariableException {

      /**
       * Wenn der beschriebene Fehler auftritt und noch die korrekte
       * Fehlermeldung inbehlt, ist der Test erfolgreich.
       */
      try {
          parser.parse(parse).gibWert(belegung);
          fail("Keine UnbekannteVariableException geworfen fuer " + parse);
          
      } catch (UnbekannteVariableException e) {
          assertEquals(invalidVar, e.gibBezeichner());
      }
  }
    
  /**
   * Startet den Test.
   *
   * @param args  wird nicht verwendet.
   */
  public static void main(String[] args) {
      junit.textui.TestRunner.run(suite());
  }
}    
