/*
 * Christoph Peters
 */

import java.util.Hashtable;

/**
 * Reprsentiert die Belegung von Variablen mit ganzen Zahlen.
 */
public class Variablenbelegung {
    
    /** Hashtable als Zuordnungsobjekt fr Variablen und deren
     *  Belegung mit Werten. */
    private Hashtable belegungen;
    
    /**
     * Erzeugt Objekte dieser Klasse mit einer leeren Variablenmenge.
     **/
    public Variablenbelegung() {
        
        this.belegungen = new Hashtable();
    }
 
    /**
     * Ordnet einer Variable mit bergebener bezeichnung einen
     * bergebenen Wert zu. Ein evtl. vorhandener alter Wert wird
     * dabei berschrieben.
     *
     * @param bezeichner  Bezeichnung der Variable.
     * @param wert        Ganzzahliger Wert der Variable.
     */
    public void belege(String bezeichner, int wert) {
        
        //der Hashtable dem Schlssel bezeichner den Wert wert zuweisen.
        belegungen.put(bezeichner, wert);
    }
    
    
    /**
     * Gibt den Wert zu einer Variable mit bergebenem Bezeichner zurck.
     * 
     * @param bezeichner  Variablenbezeichnung, zu der der Wert
     *                    zurckgegeben werden soll.
     * @return            Den Wert der in bezeichner hinterlegten Variable.
     * @throws UnbekannteVariableException  wird geworfen, wenn die bergebene
     *                    Variable unbekannt ist, ihr also noch nicht mit
     *                    belege() ein Wert zugewiesen wurde.
     */
    public int gibWert(String bezeichner) 
                    throws UnbekannteVariableException {
        
        //wert zum passenden schlssel bezeichner aus der Hashtable lesen
        Object wert = belegungen.get(bezeichner);
        
        /* wenn der schlssel in der Hashtable nicht existiert, wurde null
         * zurckgegeben. In diesem Fall wird eine 
         * UnbekannteVariableException geworfen. */
        if (wert == null) {
            throw new UnbekannteVariableException(bezeichner);
        }
        
        /* Textuelle Darstellung des Objektes Wert in eine Integer wandeln
         * und zurck geben. wert kann an dieser Stelle nicht null sein und
         * ist von natur als ein integer, das mit belege() nur integer Werte
         * hinzugefgt werden. */
        return Integer.parseInt(wert.toString());
    }
}
